
def _validate_iterable(iterable):
    """Return the argument if it is iterable, ``TypeError`` otherwise"""
    try:
        iter(iterable)
    except TypeError as e:
        raise TypeError("Argument is not iterable : %s" % repr(e))
    return iterable


def _build_iterable(iterable):
    iterable = [] if iterable is None else iterable
    if isinstance(iterable, float) or isinstance(iterable, int):
        iterable = range(int(iterable))
    return _validate_iterable(iterable)




class ColorValueError(ValueError):
    """Exception when color is not available for ColourPen"""
    pass


class StyleValueError(ValueError):
    """Exception when style is not available for ColourPen"""
    pass


class NoDurationsError(ValueError):
    """Exception when durations were not computed in Chronos"""
    pass


class DelayTypeError(TypeError):
    """Exception when no delay was given to Timer"""
    pass




import logging
import logging.config
import os

from pyux import __LOGGING_CONFIG__
from pyux import __LAUNCH_TIME__


def init_logger(folder: str = './logs',
                filename: str = 'activity',
                run_name: str = 'default',
                time_format: str = "%Y%m%d-%Hh%Mm%Ss",
                config_file = None):
    """Return a logger instance with predefined or custom format
    
    A different log file is saved at each execution. The logger is formatted
    with a default ``logging.conf`` when ``config_file`` is not provided.
    
    Log file name is of the form ``filename_run_name_time-format``. Default
    values yield, for instance : ``logs/activity_default_20190721-18h34h20s.log``. You can
    cheat with ``time_format`` by specifying a normal word.
    
    To use the logger across submodule, it is advised to instantiate the logger in
    the main script with ``logger = init_logger()``, then instantiate it in other
    modules with ``logger = logging.getLogger(__name__)``. This will not duplicate
    logging instances and will display the modules from which the logger was called
    in the logged message.
    
    :param folder: default ``'./logs'`` : folder to save log files in, created if does not exist
    :type folder: str
    :param filename: default ``'activity'`` : name of log files (first to appear)
    :type filename: str
    :param run_name: default ``'default'`` : name of run
    :type: run_name: str
    :param time_format: default ``'%Y%m%d-%Hh%Mm%Ss'`` : time format for date
    :type time_format: str
    :param config_file: default ``None`` : path to logging.conf configuration file
    :type config_file: str
    
    :return: A 'root' logger from ``logging`` package
    """
    config_file = __LOGGING_CONFIG__ if config_file is None else config_file
    log_filename = filename + '_' + run_name + '_' + __LAUNCH_TIME__.strftime(time_format)
    log_path = os.path.join(folder, "%s.log" % log_filename)
    
    try:
        logging.config.fileConfig(config_file, defaults = {'logfilename': log_path})
    except FileNotFoundError:
        os.makedirs(folder, exist_ok = True)
        logging.config.fileConfig(config_file, defaults = {'logfilename': log_path})
    return logging.getLogger('root')




from shutil import rmtree
from shutil import get_terminal_size
from time import sleep

from pyux.console import ColourPen
from pyux.console import ProgressBar
from pyux.console import Wheel
from pyux.logging import init_logger
from pyux.time import Chronos
from pyux.time import Timer


def _title_char(x):
    x = "\n===   %s   " % x
    x = x + '=' * (__width__ - len(x)) + '\n'
    return x


__pen__ = ColourPen()
__width__ = get_terminal_size()[0]

__welcome_message__ = """%s
    Welcome to pyux tools demo
%s""" % ("=" * __width__, "=" * __width__)
__intro__ = """
    All available tools in this package will now be demonstrated !
    Read the README to see the nitty gritty of using them."""

__title1__ = _title_char('1. Module console')
__intro1__ = """
    Classes available in console package help you with printing stuff
    when a script is running."""
__div_wheel__ = """
    Wheel : print a turning wheel within a for loop (or manually).
    The iteration counter or the iterated value is printed next to the wheel.
"""
__div_progress__ = """
    ProgressBar : inspired by tqdm package, print a progress bar
    within a for loop (or manually).
"""
__div_colour__ = """
    ColourPen : colourise and style text for terminal printing, with package
    colorama. This is what this demo uses to colour text !
"""

__title2__ = _title_char('2. Module time')
__intro2__ = """
    Classes available in time package are relative to time in a script : either
    making it wait, or timing durations."""
__div_timer__ = """
    Timer : print a timer for a given delay.
    A counter and a message can be printed too.
"""
__div_chronos__ = """
    Chronos : a stopwatch for your script or your loops.
    Can save results in a tsv file.
"""

__title3__ = _title_char('3. Module logging')
__intro3__ = """
    The logging module contains only function init_logger, that you can use to
    initiate a logger instance. You can specify name and destination folder for
    the log file. By defaut, the name changes at each execution so it creates
    a different log file by execution. You can give your own logging.conf or
    use the default format included."""
__div_logger__ = """
    init_logger : initiate a root logger that creates a different log file for
    each execution.
"""

__title_style__ = {'color': 'yellow', 'style': 'bright'}
__div_style__ = {'color': 'green', 'style': 'normal'}

__goodbye__ = """
    This demo has ended ! Thanks for using pyux and feel free to suggest
    improvements at https://gitlab.com/roamdam/pyux."""


def _user_yes(skip_char = 'p', action = 'pass'):
    global __pen__
    __pen__.write(color = 'white', style = 'dim')
    text = input('    Type any key to continue or %s to %s : ' % (skip_char, action))
    __pen__.write(style = 'reset_all')
    return False if text == skip_char else True


def _user_exit():
    global __pen__
    __pen__.write('\n    Do you want to continue the demo ?', color = 'red', newline = True, reset = True)
    if not _user_yes(skip_char = 'q', action = 'quit'):
        print('\n    Goodbye !')
        exit(0)
    else:
        return


def main():
    global __pen__
    __pen__\
        .write(message = __welcome_message__, style = 'bright', color = 'cyan', newline = True, reset = True)\
        .write(message = __intro__, newline = True)\
        
    __pen__\
        .write(message = __title1__, color = __title_style__['color'], style = __title_style__['style'], reset = True)\
        .write(message = __intro1__, newline = True)\
        .write(message = __div_wheel__, color = __div_style__['color'], style = __div_style__['style'], reset = True)
    if _user_yes():
        print("""
    You can print the number of iterations, here for 30 iterations""")
        for _ in Wheel(30):
            sleep(0.1)
        print("""
    Or the iterated value, here for an iteration over 30 paths""")
        sleep(2)
        paths = ['fakepath_%d' % index for index in range(30)]
        for _ in Wheel(paths, print_value = True):
            sleep(0.1)
    
    __pen__.write(message = __div_progress__, color = __div_style__['color'],
                  style = __div_style__['style'], reset = True)
    if _user_yes():
        print("""
    You can print a progress bar that will adjust to the window size,
    here for 2 500 iterations""")
        for _ in ProgressBar(2500):
            sleep(0.001)
    
    __pen__.write(message = __div_colour__, color = __div_style__['color'],
                  style = __div_style__['style'], reset = True)
    if _user_yes():
        print('\n    You can set colour and styles among predefined values :')
        __pen__\
            .write(message = '    This is a normal red', color = 'red', newline = True)\
            .write(message = '    This is a bright blue', color = 'cyan', style = 'bright', newline = True)\
            .write(reset = True)
        
        __pen__\
            .write('    You can also', color = 'green')\
            .write(' change colors within', color = 'magenta')\
            .write(' the same line !', color = 'yellow', reset = True, newline = True)
    _user_exit()

    # Section 2
    __pen__ \
        .write(message = __title2__, color = __title_style__['color'], style = __title_style__['style'], reset = True) \
        .write(message = __intro2__, newline = True) \
        .write(message = __div_timer__, color = __div_style__['color'], style = __div_style__['style'], reset = True)
    if _user_yes():
        print("\n    Use Timer to make a program wait and see where it is at.")
        Timer(delay = 5, message = "This is an example of a 5 seconds timer")
        print("""
    You can use also it for each iteration in a loop by decorating
    the iterable in a for statement.
    Here for a three iterations loop, pausing 3 seconds each time,
    and rewriting on the same line in terminal for each iteration
    (useful for big loops).""")
        sleep(8)
        for _ in Timer(3, delay = 3, pattern = 'iteration : ', overwrite = True):
            pass

    __pen__.write(message = __div_chronos__, color = __div_style__['color'],
                  style = __div_style__['style'], reset = True)
    if _user_yes():
        print("""
    Use Chronos to time stages of your script.
    You can also use it to time iterations within loops by decorating
    the iterable in the for statement.
    Here for a three iterations loop, with a pause equal to the iteration
    value.""")
        sleep(10)
        for _ in Chronos(range(1, 4), console_print = True):
            Timer(delay = _, message = "Iteration %d, waiting %d secs" % (_, _), overwrite = True)
    
    _user_exit()

    # Section 3
    __pen__ \
        .write(message = __title3__, color = __title_style__['color'], style = __title_style__['style'], reset = True) \
        .write(message = __intro3__, newline = True) \
        .write(message = __div_logger__, color = __div_style__['color'],
               style = __div_style__['style'], reset = True)
    if _user_yes():
        print("""
    A log file will be created in './tmp' with name demo_run_time-of-run.log.
    By default, the logger writes to file and to console (which can be coloured).""")
        logger = init_logger(folder = './tmp', filename = 'demo', run_name = 'run')
        sleep(5)
        logger.info('An info message without color.')
        sleep(1)
        __pen__.write(style = 'bright', color = 'red', newline = False)
        logger.critical('A critical message colored in bright red (in console only).')
        __pen__.write(
            message = "\n    A log file was created in './tmp'. Do you want to delete that folder ?",
            style = 'reset_all', newline = True
        )
        if _user_yes(skip_char = 'k', action = 'keep'):
            rmtree('./tmp')
            print("    ./tmp folder deleted !")
        
    __pen__.write(__goodbye__, color = 'yellow').close()
    

if __name__ == '__main__':
    main()



import csv
import timeit
import sys
from os.path import exists as path_exists

from time import sleep
from pyux.errors import DelayTypeError
from pyux.errors import NoDurationsError
from pyux.__utils__ import _build_iterable


class Timer:
    """Print a descending timer for a given delay

    A message can be printed next to the timer.
    The class can be used on the iterable of a for
    loop to wait the same amount of time at each iteration.
    
    :param iterable: default ``None``  – *for decorator use only* : an iterable,
        or an integer for standard ``range(n)`` values
    :type iterable: iterable or int
    :param delay: default ``None`` : time to wait, must be provided. Default
        to seconds, can be milliseconds with ``ms = True``
    :type delay: int
    :param message: default ``''`` : message to print when used manually
    :type message: str
    :param ms: default ``False`` : use a delay in milliseconds rather than seconds
    :type ms: bool
    :param pattern: default ``''`` – *for decorator use only* : prefix to print before iterated value when
        used as a loop decorator and ``print_value = False``
    :type pattern: str
    :param print_value: default ``False`` – *for decorator use only* : Print running value of the iterable
        when used to decorate a loop instead of iteration index
    :type print_value: bool
    :param overwrite: default ``False`` : do not print a new line when the timer
        is finished (useful almost only when used within a loop)
    :type overwrite: bool
    
    When used within a for loop, default is to print the
    iteration index next to the timer. You can add a constant
    string prefix to it using argument ``pattern`` (which
    is not used otherwise), or use ``print_value = True`` to
    print the running value of the iterable, instead of the iteration number.
    
    Specifying a value for ``iterable`` while calling ``Timer()`` outside of a for
    statement will have no effect.
    
    :raises: ``DelayTypeError`` if no or wrong type delay is given
    
    :Example:
    
    >>> # One-shot usage
    >>> Timer(delay = 10, message = "Waiting for 10 seconds")
    >>> # Usage as a decorator
    >>> for index in Timer(3, delay = 3):
    >>>     pass
    
    """
    def __init__(self,  iterable = None, delay: int = None,
                 message: str = '', ms: bool = False,
                 pattern: str = '', print_value: bool = False,
                 overwrite: bool = False) -> None:
        self.delay = delay
        
        # Attributes for iteration only
        self._sleep_time = 1 / 1000 if ms else 1
        self._print_value = print_value
        self._pattern = pattern
        self.iterable = _build_iterable(iterable)
        
        # If iterable is generator, it doesn't have a len, and will be launched
        # with __iter__
        try:
            len(self.iterable)
        except TypeError:
            return
        # Launch only when self.iterable = [], otherwise it will be launched
        # with __iter__
        if len(self.iterable) == 0:
            self._launch(
                delay = delay,
                message = message,
                overwrite = overwrite
            )
        return
    
    def __iter__(self):
        self.n = 0
        for element in self.iterable:
            yield element
            message = element if self._print_value else '%s%d' % (self._pattern, self.n)
            self.n += 1
            self._launch(delay = self.delay, message = message, overwrite = True)
        print('')
        
    def _launch(self, delay, message, overwrite) -> None:
        """Launch the timer for the given delay"""
        try:
            values = range(delay)
        except TypeError:
            raise DelayTypeError('No or wrong type value provided for argument `delay`')
        for n in values:
            remaining_time = str(delay - n).zfill(len(str(delay)))
            self.print(time = remaining_time, message = message)
            sleep(self._sleep_time)
        self.close(message = message, overwrite = overwrite)
        return

    @staticmethod
    def print(time: str, message: str) -> None:
        """Print the counter for a given time, counter and message"""
        sys.stdout.write('\r%s | %s' % (time, message))
        sys.stdout.flush()
        return
    
    def close(self, message: str, overwrite: bool) -> None:
        """Print ``'0'`` when time has passed (last iteration is ``1``)"""
        self.print(time = '0', message = message)
        if not overwrite:
            sys.stdout.write('\n')
            sys.stdout.flush()
        return


class Chronos:
    """Time chunks of code in a script
    
    The class works as a stopwatch during a race on track : after starting it,
    for each *lap*, a button is pushed that records time at this moment, using
    ``timeit.default_timer()``.

    Those times have not a meaning by themselves since they do not correspond 
    to durations and depends on the previous recorded times. When you want to
    know duration of recorded laps, use ``compute_durations()``, which gives
    durations for each lap and total duration.
    
    Results can be exported in a tsv file with ``write_tsv()``.
    
    :param iterable: default ``None`` – *for decorator use only* : an iterable,
        or an integer for standard ``range(n)`` values.
    :type iterable: iterable or int
    :param ms: default ``False`` : compute durations in milliseconds rather than seconds
    :type ms: bool
    :param console_print: default ``False`` – *for decorator use only* : print durations in tsv format
        in console when loop is finished
    :type console_print: bool
    :param pattern: default ``'iteration '`` – *for decorator use only* : pattern for naming laps, will be
        suffixed with the iteration number
    :type pattern: str
    :param write_tsv: default ``False`` – *for decorator use only* : write out tsv file end loop is finished
    :type write_tsv: bool
    :param run_name: default ``'default'`` – *for decorator use only* : name to give to execution for column Execution
    :type run_name: str
    :param path: default ``None`` – *for decorator use only* : full path to file to write
    :type path: str
    :param col_names: default ``None`` – *for decorator use only* : column names (length 3)
    :type col_names: tuple
    
    Chronos can be used as a decorator in a for loop. In that case, it records
    duration for each iteration. Since you cannot get the object back, you can
    either print the values to console (in tsv format) or write out the values
    in a tsv file.
    
    Specifying an ``iterable`` argument while calling ``Chronos()`` outside of a for
    statement has no effect. In that case, you can still use it manually, as if
    you entered nothing for that argument.
    
    :Example:
    
    >>> from time import sleep
    >>> chrono = Chronos()
    >>> sleep(2)
    >>> chrono.lap(name = 'lap 1')
    >>> sleep(5)
    >>> chrono.lap(name = 'lap 2')
    >>> chrono.compute_durations().durations
    >>> # In a for loop
    >>> for index in Chronos(10, console_print = True, write_tsv = False, ms = True):
    >>>     pass
    
    """
    def __init__(self, iterable = None, console_print: bool = False,
                 pattern: str = 'iteration ', write_tsv: bool = False,
                 run_name: str = 'default', path: str = None, col_names = None, ms: bool = False):
        self.lap_times = [timeit.default_timer()]
        self.lap_names = ['start']
        self.durations = {}
        self.results = None
        
        # Attributes for iteration only
        self._iterable = _build_iterable(iterable)
        self._print = console_print
        self._pattern = pattern
        self._path = path
        self._run_name = run_name
        self._write_tsv = write_tsv
        self._colnames = col_names
        self._ms = ms
        
    def __iter__(self):
        self.n = 0
        for element in self._iterable:
            yield element
            self.lap(name = '%s%d' % (self._pattern, self.n))
            self.n += 1
        self.compute_durations(ms = self._ms)
        if self._write_tsv:
            self.write_tsv(
                run_name = self._run_name,
                path = self._path,
                col_names = self._colnames
            )
        if self._print:
            print('\nStep\tDuration')
            for step, duration in self.durations.items():
                print('%s\t%d' % (step, duration))
    
    def __repr__(self):
        return 'Chronos(laps = %s)' % ', '.join(self.lap_names)
    
    def lap(self, name: str):
        """Record time for this lap, a name must be provided"""
        self.lap_names.append(name)
        self.lap_times.append(timeit.default_timer())
        return self
    
    def compute_durations(self, ms: bool = False):
        """Compute laps duration

        Duration is the difference between two adjacent laps. Results
        are stored in ``self.durations``, in seconds by default. They can be stored
        in milliseconds. The total duration is also calculated.

        :param ms: Express durations in milliseconds rather than seconds
        :type ms: bool

        :return: ``self``
        """
        for i in range(len(self.lap_names)):
            if i == 0:
                pass
            else:
                duration = self.lap_times[i] - self.lap_times[i - 1]
                if ms:
                    duration = round(duration * 1e3, 0)
                else:
                    duration = round(duration, 3)
                self.durations[self.lap_names[i]] = duration
        self.durations['total'] = sum(self.durations.values())
        return self
    
    def write_tsv(self, run_name: str, path: str, col_names: tuple = None):
        """Export durations in tsv file
        
        Write three columns, the first containing ``run_name`` : a string describing
        which execution the durations came from. This way you can append several
        execution times to the same file.
        
        Default values for column names are : Execution, Step, Duration (secs)
        
        :param run_name: name to give to execution for column Execution
        :type run_name: str
        :param path: full path to file to write
        :type path: str
        :param col_names: default ``None`` : column names of length 3
        :type col_names: tuple
        
        :return: ``self``
        """
        if len(self.durations) == 0:
            raise NoDurationsError('durations attribute is empty (have you compute_durations before ?')
        if col_names is None:
            col_names = ('Execution', 'Step', 'Duration (secs)')
        elif len(col_names) < 3 or len(col_names) > 3:
            raise ValueError('column names must be of length 3, was %s' % repr(col_names))
        table = [(run_name, step, str(time)) for step, time in self.durations.items()]
        
        file_exists = path_exists(path)
        with open(path, 'a') as out_table:
            tsv_writer = csv.writer(out_table, delimiter = '\t')
            if not file_exists:
                tsv_writer.writerow(col_names)
            for row in table:
                tsv_writer.writerow(row)
                
        return self




import shutil
import sys

from colorama import init, deinit
from colorama import Fore, Style

from pyux.errors import ColorValueError
from pyux.errors import StyleValueError
from pyux.__utils__ import _build_iterable


class Wheel:
    """Print a wheel turning at the same speed as iterations

    This class decorates an iterable in a for statement by printing a turning wheel
    and the iteration index at each step.

    :param iterable: default ``None`` : an iterable, or an integer for
        standard ``range(n)`` values
    :type iterable: iterable or int
    :param print_value: default ``False`` : print iteration value instead of
        iteration index next to the wheel
    :type print_value: bool
    
    Wheel can also be used manually (without iterable), which is useful if you
    need to print messages different than iteration value or index. It is compatible
    with generators (which makes it a good indicator that something is happening
    when you do not know the end in advance).
    
    :raise: ``TypeError`` if ``iterable`` is not an iterable
    
    :Example:
    
    >>> for _ in Wheel(['a', 'b', 'c', 'd']):    # print the index at each step next to the weel
    >>> for _ in Wheel(5):                       # same behaviour than previous
    >>> for letter in Wheel(['a', 'b', 'c', 'd'], print_value = True):
    >>>     # this whill print the iteration value (letters)
    >>>     # rather than the iteration index
    
    """
    def __init__(self, iterable = None, print_value: bool = False):
        self.print_value = print_value
        self.iterable = _build_iterable(iterable)
        self.last_message = None
        
    def __iter__(self):
        self.n = 0
        for element in self.iterable:
            yield element
            if self.print_value:
                self.print(step = self.n, message = element)
                self.last_message = element
            else:
                self.print(step = self.n)
                self.last_message = self.n + 1
            self.n += 1
        self.close(message = self.last_message)
        
    @staticmethod
    def print(step: int, message: str = None) -> None:
        """Print the wheel for a given step and message"""
        message = step if message is None else message
        if step % 4 == 0:
            sys.stdout.write('\r - | %s' % message)
            sys.stdout.flush()
        elif step % 4 == 1:
            sys.stdout.write('\r \\ | %s' % message)
            sys.stdout.flush()
        elif step % 4 == 2:
            sys.stdout.write('\r | | %s' % message)
            sys.stdout.flush()
        else:
            sys.stdout.write('\r / | %s' % message)
            sys.stdout.flush()
        return
    
    def close(self, message: str = None) -> None:
        """Print the last iteration wheel and flush console
        
        This method makes the last printed value equal to the
        number of iterations. If a message is provided, it is printed
        instead of the former.
        
        :param message: default ``None`` : message to print next to wheel
        :type message: str
        
        It is automatically called when the method is used as a decorator of
        a for statement.
        
        When Wheel works with no iterable provided (which is internally considered as
        an iterable of length zero), it will logically print `0` when closing.
        A workaround is to use ``close(message = '')`` if you want the last printed
        value to stay, or any other message if you want it to be replaced by an
        ending message.
        
        :Example:
        
        >>> wheel = Wheel(iterable = 'A string iterable')
        >>> wheel.close()
        >>> wheel.close(message = 'Overriding length of iterable')
        
        """
        try:
            last_n = len(self.iterable)
        except TypeError:
            last_n = self.n
        if message is not None:
            self.print(step = last_n, message = message)
        else:
            self.print(step = last_n)
        sys.stdout.write('\n')
        sys.stdout.flush()
        return
    

class ProgressBar:
    """Print progress bar with percent and number of iterations

    ProgressBar is meant to be used as a decorator in a for statement :
    for each iteration, it prints the percentage of progress, the number of iterations
    against the total number of iterations to do, and a progress bar.
    
    :param iterable: default ``None`` : an iterable, or an integer for standard ``range(n)`` values
    :type iterable: iterable or int
    :param ascii_only: Use ascii character ``=`` to print the bar
    :type ascii_only: bool
    
    A ProgressBar can also be created manually if called outside of a for
    statement. However it still *needs* an iterable to know the total number of
    iterations. This means it cannot be used with generators.
    
    A workaround if you really want to use it even if you don't know how long
    the iteration will go, give an approximation as an integer to ``iterable``
    argument. This might cause unwanted behaviour in console, especially if your
    approximation is too short.
    
    :raise: ``TypeError`` if ``iterable`` is not an iterable
    
    :Example:
    
    >>> from time import sleep
    >>> for _ in ProgressBar(3000, ascii_only = True):
    >>>     sleep(0.001)
    
    """
    def __init__(self, iterable = None, ascii_only: bool = False):
        self.iterable = _build_iterable(iterable)
        self.bar_char = '=' if ascii_only else '\u2588'
        
        self.real_width = len(self.iterable)
        give_space = 4 + 3 + len(str(self.real_width))*2 + 1 + 3 + 2    # xxx% | step/total_step | ===... |
        self.bar_width = shutil.get_terminal_size()[0] - give_space
        self.real_step = self.real_width / self.bar_width
        
        self.current_console_step = 0
        
    def __repr__(self):
        return "ProgressBar(real_width = %d)" % self.real_width
        
    def __iter__(self):
        self.n = 0
        for element in self.iterable:
            yield element
            self.print(step = self.n)
            self.n += 1
        self.close()
        
    def print(self, step: int) -> None:
        """Print progress bar for the current iteration"""
        if step >= self.current_console_step * self.real_step:
            str_values = (
                step * 100 / self.real_width,
                str(step).zfill(len(str(self.real_width))),
                self.real_width,
                self.bar_char * self.current_console_step
            )
            sys.stdout.write('\r%d%% | %s/%d | %s' % str_values)
            sys.stdout.flush()
            self.current_console_step += 1
        return
    
    def close(self) -> None:
        """Print bar with 100% completion

        When the for loop has finished, print a last progress bar with 100%
        completion.
        
        This method is automatically called at the end of a for loop when
        ``ProgressBar`` decorates a for statement.
        
        Without closing, iteration index would end just before reaching 100%
        due to index starting from zero rather than one.
        
        :Example:
        
        >>> ProgressBar(iterable = 'A string iterable').close()
        
        """
        sys.stdout.write('\r100%% | %d/%d | ' % (self.real_width, self.real_width))
        sys.stdout.write(self.bar_char * self.bar_width + ' |\n')
        sys.stdout.flush()
        return


class ColourPen:
    """Util for writing colored and styled messages in console

    Available colors and styles can be found in ``ColourPen.__colors__``
    and ``ColourPen.__styles__``. Arguments are matched against these values.
    
    Only one instance is needed to print text with different styles. By default,
    styles and colours remain the same until changed or reset, allowing to
    write successive prints with the same format, without having to specify it
    each time.
    
    :Example:
    
    >>> pen = ColourPen()
    >>> pen.write('A blue and bright message', color = 'cyan', style = 'bright')
    >>> pen.write('Still a blue message', reset = True, newline = True)
    >>> pen.write('That message is back to normal')
    >>> pen.close()
    
    """
    __colors__ = [
        'BLACK',
        'RED',
        'GREEN',
        'YELLOW',
        'BLUE',
        'MAGENTA',
        'CYAN',
        'WHITE',
        'RESET'
    ]
    __styles__ = [
        'DIM',
        'NORMAL',
        'BRIGHT',
        'RESET_ALL'
    ]
    
    def __init__(self):
        init()
        self.message = ''
    
    def __repr__(self):
        return "ColourPen(message = %s)" % self.message
    
    def _colourise(self, color = 'RESET'):
        """Colourise the message with the given color

        Color argument is checked against possible values and
        is case insensitive. Can be combined with style.
        
        :param color: default ``'RESET'`` : color to use. Giving ``None`` does
        nothing, which is, returns the message with the color from the
        previous call to ``write()``
        :type color: str
        
        :raises: ``ColorValueError`` if the color does not exist
        """
        try:
            color = color.upper()
        except AttributeError:
            pass
        try:
            color_prefix = getattr(Fore, color)
        except TypeError:
            color_prefix = ''
        except AttributeError:
            raise ColorValueError("color must be one of %s" % repr(self.__colors__))
        self.message = color_prefix + self.message
        return self
    
    def _style(self, style = 'RESET_ALL'):
        """Style the message with the given style

        Style argument is checked against possible values and
        is case insensitive. Can be combined with colourise.
        
        :param style: default ``'RESET_ALL'`` : style to use. Giving ``None`` does
        nothing, which is, returns the message with the style from the
        previous call to ``write()``
        :type style: str
        
        :raises: ``StyleValueError`` if the style does not exist
        """
        try:
            style = style.upper()
        except AttributeError:
            pass
        try:
            style_prefix = getattr(Style, style)
        except TypeError:
            style_prefix = ''
        except AttributeError:
            raise StyleValueError("style must be one of %s" % repr(self.__styles__))
        self.message = style_prefix + self.message
        return self

    def write(self, message: str = '',
              color: str = None, style: str = None,
              flush: bool = True, newline: bool = False, reset: bool = False):
        """Write a colored and styled message
        
        The method returns ``self`` so that you can chain calls to the method
        from a single instance.
        
        :param message: default ``''`` : message to write
        :type message: str
        :param color: default ``None`` : colour to use, checked against possible values.
        :type color: str
        :param style: default ``None`` : style to use, checked against possible values
        :type style: str
        :param flush: default ``True`` : flush console after writing message
        :type flush: bool
        :param newline: default ``False`` : insert a new line after writing message
        :type newline: bool
        :param reset: default ``False`` : reset style and colour after writing message
        :type reset: bool
        
        Default behavior is to pass on current style to subsequent calls, so you
        only need to style and colour once if you want to keep the same format. This
        is accomplished through ``None`` values for ``color`` and/or ``style``,
        which means that giving ``None`` does not ignore style and color !
        
        To remove style for the message to print, use ``style = 'RESET_ALL'``.
        To remove style after printing, use ``reset = True``
        
        Message is printed with ``sys.stdout.write()`` rather than ``print()``,
        allowing precise control over printing. Default behavior is to flush at
        each print and to keep the console cursor where it is.
        
        You can thus decide to "write" several messages and flush them at once,
        and you can add a newline after writing to mimic the behavior of ``print()``.
        
        :return: ``self``
        :raises: ``ColorValueError``, ``StyleValueError``

        :Example:
        
        >>> pen = ColourPen()
        >>> pen.write('Hello... ', color = 'cyan').write('Goodbye !', reset = True, newline = True)

        """
        self.message = message
        self._colourise(color = color)\
            ._style(style = style)
        sys.stdout.write(self.message)
        
        if reset:
            self.write(message = '', style = 'RESET_ALL', reset = False, newline = newline)
            return self
        if newline:
            self.write(message = '\n', newline = False, reset = reset)
            return self
        if flush:
            sys.stdout.flush()
        return self
        
    def close(self) -> None:
        """Reset all styles and close ``colorama`` util
        
        The method flushes an empty message with reset styles and a new line.
        Closing make the ``sys.stdout`` go back to normal : styling and colouring
        will not be recognised after it.
        """
        self.write(message = '', flush = True, reset = True, newline = True)
        deinit()
        return



import setuptools

with open('requirements.txt', 'r') as file:
    requirements = file.read().splitlines(keepends = False)

with open('README.rst', 'r') as file:
    README = file.read()

setuptools.setup(
    name='pyux-track',
    version='0.1.1',
    description="Python ux tools to ease tracking scripts",
    long_description=README,
    long_description_content_type="text/x-rst",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    include_package_data=True,
    author='Romain Damian',
    author_email='damian.romain@gmail.com',
    url='https://pyux.readthedocs.io/en/stable/'
)
